package com.fjhx.sale.service.contract.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.flow.service.flow.FlowExampleService;
import com.fjhx.item.entity.product.po.ProductInfo;
import com.fjhx.item.service.product.ProductClassifyService;
import com.fjhx.item.service.product.ProductInfoService;
import com.fjhx.purchase.entity.purchase.vo.PurchaseDetailVo;
import com.fjhx.purchase.service.purchase.PurchaseDetailService;
import com.fjhx.sale.entity.contract.dto.ContractProductDto;
import com.fjhx.sale.entity.contract.dto.ContractProductSelectDto;
import com.fjhx.sale.entity.contract.po.ContractProduct;
import com.fjhx.sale.entity.contract.po.ContractProductBom;
import com.fjhx.sale.entity.contract.vo.ContractProductBomVo;
import com.fjhx.sale.entity.contract.vo.ContractProductVo;
import com.fjhx.sale.mapper.contract.ContractProductMapper;
import com.fjhx.sale.service.contract.ContractProductBomService;
import com.fjhx.sale.service.contract.ContractProductService;
import com.fjhx.tenant.utils.DeptUstil;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.wrapper.IWrapper;
import com.ruoyi.system.utils.UserUtil;
import org.apache.commons.collections4.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;


/**
 * <p>
 * 外销合同-产品 服务实现类
 * </p>
 *
 * @author _
 * @since 2023-04-17
 */
@Service
public class ContractProductServiceImpl extends ServiceImpl<ContractProductMapper, ContractProduct> implements ContractProductService {

    @Autowired
    private ProductInfoService productInfoService;
    @Autowired
    private ProductClassifyService productClassifyService;
    @Autowired
    private PurchaseDetailService purchaseDetailService;
    @Autowired
    private FlowExampleService flowExampleService;
    @Autowired
    private ContractProductBomService contractProductBomService;

    /**
     * 分页
     */
    @Override
    public Page<ContractProductVo> getPage(ContractProductSelectDto dto) {
        IWrapper<ContractProduct> wrapper = getWrapper();
        wrapper.orderByDesc("t1", ContractProduct::getCreateTime);
        if (StringUtils.isNotEmpty(dto.getKeyword())) {

            //关键字检索
            if (ObjectUtil.isNotEmpty(dto.getKeyword())) {
                List<Long> pIds = productInfoService.listObject(ProductInfo::getId, q -> q
                        .like(ProductInfo::getCustomCode, dto.getKeyword())
                        .or().like(ProductInfo::getName, dto.getKeyword())
                );
                wrapper.and(q -> q
                        .in("t1.product_id", pIds)
                        .or().like("t2.`code`", dto.getKeyword())
                        .or().like("t2.`user_name`", dto.getKeyword())
                );
            }
        }
        //是否到账过滤
        if (ObjectUtil.isNotEmpty(dto.getIsReceived())) {
            if (1 == dto.getIsReceived()) {
                wrapper.isNotNull("co.id");
            } else {
                wrapper.isNull("co.id");
            }
        }
        wrapper.groupBy("t1.id");
        Page<ContractProductVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        List<ContractProductVo> records = page.getRecords();
        if (records.size() == 0) {
            return page;
        }

        // 赋值产品属性
        productInfoService.attributeAssign(records, ContractProductVo::getProductId, (item, product) -> {
            item.setProductCategoryId(product.getProductClassifyId());

            item.setProductCode(product.getCustomCode());
            item.setProductName(product.getName());
            item.setProductUnit(product.getUnit());
            item.setProductSpec(product.getSpec());
            item.setProductLength(product.getLength());
            item.setProductWidth(product.getWidth());
            item.setProductHeight(product.getHeight());
            item.setProductColor(product.getColor());
        });
        // 赋值产品分类名称
        productClassifyService.attributeAssign(records, ContractProductVo::getProductCategoryId, (item, productClassify) -> item.setProductCategory(productClassify.getName()));

        // 赋值业务员名称
        UserUtil.assignmentNickName(records, ContractProductVo::getSalesmanId, ContractProductVo::setUserName);

        // 赋值流程id
        flowExampleService.setFlowId(records, ContractProductVo::getContractId, ContractProductVo::setFlowId);


        //获取生产公司信息
        List<Long> companyIds = records.stream().map(ContractProductVo::getCompanyId).collect(Collectors.toList());
        Map<Long, String> companyNameMap = DeptUstil.getDeptNameMap(companyIds);
        for (ContractProductVo record : records) {
            //赋值生产公司名称
            record.setCorporationName(companyNameMap.get(record.getCompanyId()));
        }


        return page;
    }

    @Override
    public List<ContractProductVo> getList(IWrapper<ContractProduct> wrapper) {
        List<ContractProductVo> list = baseMapper.getList(wrapper);
        for (ContractProductVo contractProductVo : list) {
            BigDecimal saleOutboundQuantity = contractProductVo.getSaleOutboundQuantity();
            if (saleOutboundQuantity == null) {
                saleOutboundQuantity = BigDecimal.ZERO;
            }
            BigDecimal notSaleOutboundQuantity = contractProductVo.getQuantity().subtract(saleOutboundQuantity);
            contractProductVo.setNotSaleOutboundQuantity(notSaleOutboundQuantity);
        }
        return list;
    }

    /**
     * 根据合同ID和客户ID查询未包装的产品
     */
    @Override
    public List<ContractProductVo> getNoPackContractProductById(String customerId, String contractIds) {
        List<ContractProductVo> list = baseMapper.getNoPackContractProductById(customerId, Arrays.asList(contractIds.split(",")));
        // 赋值产品属性
        productInfoService.attributeAssign(list, ContractProductVo::getProductId, (item, product) -> {
            item.setProductCode(product.getCode());
            item.setProductUnit(product.getUnit());
            item.setProductType(product.getType());
            item.setProductName(product.getName());
            item.setProductSpec(product.getSpec());
            item.setProductCategoryId(product.getProductClassifyId());
        });
        // 赋值产品分类名称
        productClassifyService.attributeAssign(list, ContractProductVo::getProductCategoryId, (item, productClassify) -> item.setProductCategory(productClassify.getName()));
        return list;
    }

    /**
     * 根据合同产品IDS查询合同产品
     */
    @Override
    public List<ContractProductVo> getListDetail(List<Long> ids) {
        if (CollectionUtils.isEmpty(ids)) {
            throw new ServiceException("参数异常");
        }
        IWrapper<ContractProduct> wrapper = getWrapper();
        wrapper.in("t1", ContractProduct::getId, ids);
        List<ContractProductVo> list = baseMapper.getListByIds(wrapper);
        if (CollectionUtils.isNotEmpty(list)) {
            List<PurchaseDetailVo> vo = purchaseDetailService.getSumCountInDataResourceIds(ids);
            Map<Long, BigDecimal> voMap = vo.stream().collect(Collectors.toMap(PurchaseDetailVo::getDataResourceId, PurchaseDetailVo::getSumCount));
            for (ContractProductVo c : list) {
                if (MapUtils.isNotEmpty(voMap)) {
                    c.setSumPurchaseCount(voMap.getOrDefault(c.getId(), BigDecimal.ZERO));
                }
            }

            // 赋值产品属性
            productInfoService.attributeAssign(list, ContractProductVo::getProductId, (item, product) -> {
                item.setProductCode(product.getCode());
                item.setProductUnit(product.getUnit());
                item.setProductType(product.getType());
                item.setProductName(product.getName());
                item.setProductSpec(product.getSpec());
                item.setProductCategoryId(product.getProductClassifyId());
            });

            // 赋值产品分类名称
            productClassifyService.attributeAssign(list, ContractProductVo::getProductCategoryId, (item, productClassify) -> item.setProductCategory(productClassify.getName()));


        }
        return list;
    }

    @Override
    public ContractProductVo detail(Long id) {
        return BeanUtil.copyProperties(this.getById(id), ContractProductVo.class);
    }

    @Override
    public void add(ContractProductDto contractProductDto) {
        this.save(contractProductDto);
    }

    @Override
    public void edit(ContractProductDto contractProductDto) {
        this.updateById(contractProductDto);
    }

    @Override
    public void delete(Long id) {
        this.removeById(id);
    }

    @Override
    public ContractProductVo reviewDetail(Long id) {
        ContractProductVo result = baseMapper.detail(id);
        List<ContractProductBom> list = contractProductBomService.list(q -> q
                .eq(ContractProductBom::getContractProductId, id)
                .eq(ContractProductBom::getType, 3)
        );
        result.setContractProductBomList(BeanUtil.copyToList(list, ContractProductBomVo.class));
        return result;
    }

}
